Gamestate = require "libs.hump.gamestate"   -- Gamestate handling
Timer = require "libs.hump.timer"           -- Timed events
Camera = require "libs.hump.camera"           -- Timed events
bump = require "libs.bump"                  -- Collision handler
mainFont = love.graphics.newFont("pixel.ttf", 12)
love.graphics.setFont(mainFont)
love.graphics.setBackgroundColor(135,206,250)
love.mouse.setVisible(false)

screen = {
    W = love.graphics.getWidth(),
    H = love.graphics.getHeight()
}

local menu = {}
local game = {}

function switchState(state)
    Gamestate.switch(state)
end

function menu:init()
title = {
    word = "pong",
    x = screen.W/2,
    y = screen.H/2,
}
title.width = mainFont:getWidth(title.word)
text = {
    description = "Press SPACE to play!",
    x = screen.W/2,
    y = screen.H/2,
}
end

function menu:update(dt)
    if love.keyboard.isDown(' ') then
        switchState(game)
    end
end

function menu:draw()
    love.graphics.print(title.word, title.x, title.y)
    love.graphics.print(text.description, text.x, text.y+20)
    love.graphics.print("2016, tt", screen.W-80, screen.H-30)
end

function game:init()
music = love.audio.newSource("assets/music.mp3")
music:play()
music:setLooping(true)

player = {
    x = 40,
    y = screen.H/2-50,
    w = 15,
    h = 100,
    yvel = 0,
    speed = 5,
    direction = 1,
    wins = false,
    name = "player",
        goal = {
            x = 0,
            y = 10,
            w = 10,
            h = screen.H-20
        }
}
ai = {
    x = screen.W-player.w-40,
    y = screen.H/2-player.h/2,
    w = player.w,
    h = player.h,
    yvel = 0,
    speed = player.speed*3,
    CanMove = false,
    wins = false,
    name = "ai",
        goal = {
            x = screen.W-10,
            y = 10,
            w = 10,
            h = screen.H-20
        }
}
ball = {
    x = screen.W/2-5, -- 1/2 of radius,center
    y = screen.H/2,
    r = 20,
    xvel = 0,
    yvel = 0,
    speed = 800,
    accelerator = 0,
    maxvel = 10,
    direction = 1,
    xvel = 0,
    yvel = 0,
    rotation = math.rad(45),
    collided = false,
    name = "ball"
}
score = {
    player = 0,
    ai = 0,
    winCount = 10
}
top = {
    name="top",
    x = 0,
    y = 0,
    w = screen.W,
    h = 10
}
bottom = {
    name="bottom",
    x = 0,
    y = screen.H-10,
    w = screen.W,
    h = 10,
}
console = {
    displayed = false
}
text = {
    size = 1
}
angleAmp = 10 -- amplify reflection angle

-- Bump init
world = bump.newWorld()
world:add(player, player.x, player.y, player.w, player.h)
world:add(ai, ai.x, ai.y, ai.w, ai.h)
world:add(ball, ball.x, ball.y, ball.r, ball.r)
world:add(top, top.x, top.y, top.w, top.h)
world:add(bottom, bottom.x, bottom.y, bottom.w, bottom.h)

-- Trailing hypespeed
positions = {}
maxPositions = 20
-- Paddle particles
local particle = love.graphics.newImage('assets/particle.png')

makeConfetti = love.graphics.newParticleSystem(particle, 32)
makeConfetti:setParticleLifetime(0.5, 1) -- Particles live at least 2s and at most 5s.
makeConfetti:setColors(255, 255, 255, 255, 255, 255, 255, 0) -- Fade to black.
makeConfetti:setSizeVariation(1)

newRound()
end

function game:enter()
    camera = Camera(screen.W/2, screen.H/2)
end

function game:update(dt)
-- Player controls
    if love.keyboard.isDown('w') or love.keyboard.isDown('up') then
        player.yvel = player.yvel + player.speed
        player.direction = -1
    elseif love.keyboard.isDown('s') or love.keyboard.isDown('down') then
        player.yvel = player.yvel + player.speed
        player.direction = 1
        else player.yvel = 0
    end
    -- Player direction (up/down)
    player.y = player.y + player.yvel * player.direction
    -- Limit paddle speed
    if player.yvel > player.speed then player.yvel = player.speed end
    -- Update player.x associated with bounding box
    world:move(player, player.x, player.y)
    -- Update the player associated bounding box in the world
    player.x, player.y, cols, len = world:move(player, player.x, player.y)

    -- Get angle of the ball relative to the center of the paddle
    function getAngle(paddle)
        local centerX, centerY = paddle.x+paddle.w/2, paddle.y+paddle.h/2
        local hypotenuse = math.floor(math.sqrt((ball.x-centerX)^2+(ball.y-centerY)^2))
        local opposite = ball.y - centerY
        local angle = math.floor(math.deg(math.sin(opposite/hypotenuse)))
        return hypotenuse, opposite, angle
    end -- return values below
    pH, pO, pA = getAngle(player)
    aH, aO, aA = getAngle(ai)

    -- Ball controls
    ball.xvel = (ball.speed * ball.direction) -- Ball direction on reset() / start
    ball.x = ball.x - ball.xvel * dt
    ball.y = ball.y - ball.yvel * dt

    -- Move ball
    local function bounce(item, other) return "bounce" end
    ball.x, ball.y, cols, len = world:move(ball, ball.x, ball.y)
    if len > 0 then ball.collided = true else ball.collided = false end
    if ball.collided == true then ball.direction = ball.direction*-1 end

    -- Limit x/y velocity of ball
    if ball.xvel > ball.speed then ball.xvel = ball.speed end
    if ball.yvel > ball.speed then ball.yvel = ball.speed end

    for i,v in ipairs (cols) do
        if cols[i].normal.y == -1 then -- Hit the bottom
            ball.yvel = -ball.yvel
            ball.direction = ball.direction * -1
        elseif cols[i].normal.y == 1 then -- Hit the top
            ball.yvel = -ball.yvel
            ball.direction = ball.direction * -1
        end
        if cols[i].normal.x ~= 0 then -- Hit player paddle
            if ball.direction == -1 then
                ball.yvel = ball.yvel - pA * angleAmp
            elseif ball.direction == 1 then -- Hit AI paddle
                ball.yvel = ball.yvel - aA * angleAmp
            end
            makeConfetti:emit(32)
            playSound(hit)
            --shakeCam()
            squashBall()
        end
    end


    -- AI controls
    -- Say when the AI is allowed to move
    if ball.x > screen.W/2 then
        ai.CanMove = true
        else ai.CanMove = false
    end
    -- Control AI movement according to ball position if over 'CanMove' line
    local aicenterY = ai.y+ai.h/2
    if ai.CanMove == true then
        if ball.y > aicenterY then
            ai.yvel = ai.yvel + ai.speed * 0.2 -- (k)onstant in place of dt
        elseif ball.y < aicenterY then
            ai.yvel = ai.yvel - ai.speed * 0.2
        elseif ball.y == ai.y then
            ai.yvel = 0
        end
    elseif
        ai.CanMove == false then
            ai.yvel = 0
    end

    -- Update y location
    if ai.CanMove == true then
        ai.y = ai.y + ai.yvel
        -- Move back to center of screen in y
    end

    -- Limit y velocity of ai
    if ai.yvel > ai.speed then ai.yvel = ai.speed end
    -- bump move ai
    ai.x, ai.y, cols, len = world:move(ai, ai.x, ai.y)

    -- Scoring
    if ball.x < 0 then
        newRound()
        score.ai = score.ai + 1
    elseif ball.x > screen.W then
        newRound()
        score.player = score.player + 1
        increaseDifficulty()
    end
    if score.player == score.winCount or score.ai == score.winCount then
        if score.player == score.winCount then winner = "Player wins!"
        elseif score.ai == score.winCount then winner = "A.I. wins!"
        end
        winnerWidth = mainFont:getWidth(winner)
        winnerHeight = mainFont:getHeight(winner)
        endgame()
    end

     -- Bump updates
    world:update(player, player.x, player.y)
    world:update(ai, ai.x, ai.y)
    world:update(ball, ball.x, ball.y)

    -- HUMP timer
   Timer.update(2*dt)

   -- Particles
   makeConfetti:update(dt)
   positionActual = { x = ball.x, y = ball.y}
   speedTrail()
   table.insert(positions, positionActual)
   if #positions > maxPositions then
       table.remove(positions, 1)
   end
end

function game:draw()
    camera:attach()
        -- Items
        love.graphics.setColor(255,64,64)
        love.graphics.rectangle("fill", player.x, player.y, player.w, player.h)
        love.graphics.rectangle("fill", ai.x, ai.y, ai.w, ai.h)
        love.graphics.setColor(255,255,255)
        love.graphics.rectangle("fill", ball.x, ball.y, ball.r, ball.r)

        -- Scores
        love.graphics.print(score.player, 20, 15, 0)
        love.graphics.print(score.ai, screen.W-30, 15)

        -- Borders
        love.graphics.rectangle("fill", top.x, top.y, top.w, top.h)
        love.graphics.rectangle("fill", bottom.x, bottom.y, bottom.w, bottom.h)
        love.graphics.print("|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|", screen.W/2)
    camera:detach()

    -- Feedback
    if console.displayed == true then
    love.graphics.print(
        "player.y = " .. player.y .. "\n" ..
        "player.yvel = " .. player.yvel * player.direction .. "\n\n\n" ..
        "ai.y = " .. math.floor(ai.y) .. "\n" ..
        "ai.yvel = " .. ai.yvel .. "\n" ..
        "ai.speed = " .. ai.speed .. "\n" ..
        "ai.h = " .. ai.h .. "\n" ..
        "ai.CanMove = " .. tostring(ai.CanMove) .. "\n\n" ..
        "ball.x = " .. math.floor(ball.x) .. "\n" ..
        "ball.y = " .. math.floor(ball.y) .. "\n" ..
        "ball.yvel = " .. ball.yvel .. "\n" ..
        "ball.xvel = " .. ball.xvel .. "\n" ..
        "ball.collision = " .. tostring(ball.collided) .. "\n" ..
        "ball.speed = " .. ball.speed .. "\n" ..
        "ball.accelerator = " .. ball.accelerator .. "\n" ..
        "ball.direction = " .. ball.direction,
        80, 22)
    love.graphics.print(
        -- Angle feedback
        "player hyp. = " .. pH .. "\n" ..
        "player opp. = " .. math.floor(pO) .. "\n" ..
        "player ang. = " .. pA .. "\n\n" ..

        "ai hyp. = " .. aH .. "\n" ..
        "ai opp. = " .. math.floor(aO) .. "\n" ..
        "ai ang. = " .. aA .. "\n\n" ..
        "hyperspeed = " .. tostring(hyperspeed) .. "\n" ..
        "showBallDirection = " .. tostring(drawBallDirection) .. "\n",
        300, 20)
        -- Lines
        love.graphics.line(player.x+player.w/2, player.y+player.h/2, ball.x+ball.r/2, ball.y+ball.r/2)
        love.graphics.line(ai.x+ai.w/2, ai.y+ai.h/2, ball.x+ball.r/2, ball.y+ball.r/2)
    end

    -- Scoring alert
    if showWinner == true then
        love.graphics.setColor(255,64,64)
        love.graphics.rectangle("fill", screen.W/2-winnerWidth/2-10, screen.H/2-winnerHeight/2-10, winnerWidth+20, winnerHeight+20 )
        love.graphics.setColor(255,255,255)
        love.graphics.print(winner, screen.W/2-winnerWidth/2, screen.H/2-winnerHeight/2)
    end

    -- Ball direction on new round
    if drawBallDirection == true then
        showBallDirection()
    end

    -- Particles
    drawConfetti()
    if ball.direction == 1 then
        makeConfetti:setLinearAcceleration(0, -400, 200, 400) -- Randomized movement towards the bottom of the screen.
    elseif ball.direction == -1 then
        makeConfetti:setLinearAcceleration(0, -400, -200, 400) -- Randomized movement towards the bottom of the screen.
    end
    if hyperspeed == true then
        for i = 1, #positions do
            love.graphics.setColor(255, 255, 255, (124/#positions)*i)
            love.graphics.rectangle("fill", positions[i].x, positions[i].y, ball.r, ball.r)
        end
    end
end

--[[ Game functions ==================================================================== ]] --

function squashBall()
    Timer.script(function(wait)
        local maxvel = math.abs(ball.yvel)
        if maxvel > 20 then maxvel = 20 end
        Timer.tween(0.5, ball, {r = 40+maxvel, y = ball.y - (40+maxvel)/2}, 'out-bounce')
        wait(0.7)
        Timer.tween(0.5, ball, {r = 20, y = ball.y + (40+maxvel)/2}, 'in-bounce')
    end)
end

function drawConfetti()
    if ball.direction == -1 then
        love.graphics.draw(makeConfetti, player.x+player.w/2, player.y+player.h/2)
    elseif ball.direction == 1 then
        love.graphics.draw(makeConfetti, ai.x+ai.w/2, ai.y+ai.h/2)
    end
end

function speedTrail()
    if ball.yvel > 550 or ball.yvel < -550 then hyperspeed = true
    else hyperspeed = false
    end
end

function shakeCam()
    local orig_x, orig_y = camera:position()
    Timer.during(0.2, function()
        camera:lookAt(orig_x + math.random(-10,10), orig_y + math.random(-10,10))
    end, function()
        -- reset camera position to screen.W/H incase of double hit offset
        camera:lookAt(screen.W/2, screen.H/2)
    end)
end

function playSound(sound)
    local blip1 = love.audio.newSource("assets/blip.wav")
    blip1:setVolume(0.2)
    local blip2 = love.audio.newSource("assets/blip2.wav")
    blip2:setVolume(0.2)
    local num = math.random(2)
    if sound == hit then
        if num == 1 then
            blip1:play()
        elseif num == 2 then
            blip2:play()
        end
    end
end

function endgame()
    showWinner = true
    player.speed = 0
   -- Timer.after(3, function() os.exit() end)
end

function newRound()
    Timer.script(function(wait)
        ball.x = screen.W/2-ball.r/2
        ball.y = screen.H/2
        ball.yvel = 0
        ball.xvel = 0
        ball.speed = 0
        player.y = screen.H/2-player.h/2
        ai.y = screen.H/2-ai.h/2
        drawBallDirection = true
            wait(1)
        ball.speed = 800 + ball.accelerator
        drawBallDirection = false
        world:remove(ai)
        world:add(ai, ai.x, ai.y, ai.w, ai.h)
    end)
end

function restart()
    ai.h = 100
    ball.speed = 800
    score.player = 0
    score.ai = 0
    ai.speed = player.speed*3
    newRound()
end

function showBallDirection()
    local arrow = love.graphics.newImage("assets/arrow.png")
    local arrowW = arrow:getWidth()
    local arrowH = arrow:getHeight()
    if ball.direction == 1 then
        love.graphics.draw(arrow, screen.W/2-50, screen.H/2+10, 0, 0.1, 0.1, arrowW/2, arrowH/2)
    elseif ball.direction == -1 then
        love.graphics.draw(arrow, screen.W/2+50, screen.H/2+10, math.rad(180), 0.1, 0.1, arrowW/2, arrowH/2)
    end
end

function increaseDifficulty()
    if drawBallDirection == true then -- really can't be bothered to make a new variable
        if score.player >= 2 then
            ai.speed = ai.speed + 2
            ball.accelerator = ball.accelerator + 50
            ai.h = ai.h + 10
        elseif score.player >= 4 then
            ai.speed = ai.speed + 2
            ball.accelerator = ball.accelerator + 50
            ai.h = ai.h + 10
        elseif score.player >= 6 then
            ai.speed = ai.speed + 2
            ball.accelerator = ball.accelerator + 50
            ai.h = ai.h + 10
        elseif score.player >= 8 then
            ai.speed = ai.speed + 2
            ball.accelerator = ball.accelerator + 50
            ai.h = ai.h + 10
        end
    elseif drawBallDirection == false then
        ball.speed = 0
    end
end

function love.keyreleased(k)
    if k == '`' then -- Toggle console
        if console.displayed == true then console.displayed = false
        elseif console.displayed == false then console.displayed = true
        end
    end
    if k == 'escape' then os.exit() end
    if k == ' ' then restart() end
end

function love.load()
    Gamestate.registerEvents()
    switchState(game)
end
